@@ -14,8 +14,6 @@
 #define __LIBMEMCACHED_MEMCACHED_H__
 
 #include <inttypes.h>
-#include <netdb.h>
-#include <netinet/in.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
@@ -26,6 +24,7 @@
 
 #include <libmemcached/visibility.h>
 #include <libmemcached/configure.h>
+#include <libmemcached/platform.h>
 #include <libmemcached/constants.h>
 #include <libmemcached/types.h>
 #include <libmemcached/string.h>
@@ -72,7 +71,7 @@ struct memcached_st {
     bool cork:1;
     bool hash_with_prefix_key:1;
     bool ketama_weighted:1;
-    bool no_block:1;
+    bool no_block:1; // Don't block
     bool no_reply:1;
     bool randomize_replica_read:1;
     bool reuse_memory:1;
